#include <mpi.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "realtime.h"

double CalcPi (int n, int iRank, int iNumProcs);

int main(int argc, char **argv)
{
    int n;
    int iMyRank, iNumProcs;
    const double fPi25DT = 3.141592653589793238462643;
    double fPi;
    double fTimeStart, fTimeEnd;

    double fLocalPi;
    /* MPI Initialization */
	/*
	   Insert calls to
	      (1) MPI_Init
	      (2) MPI_Comm_size: get size of MPI_COMM_WORLD in iNumProcs
	      (3) MPI_Comm_rank: get rank in MPI_COMM_WORLD in iMyRank
	*/
    
#ifdef READ_INPUT  
    if (iMyRank == 0) 
    {
        printf("Enter the number of intervals: ");
        scanf("%d",&n);
        if (n <= 0 || n > 2147483647 ) 
        {
            printf("\ngiven value has to be between 0 and 2147483647\n");
            return 1;
        }
    }
#else
    n = 150000000;
#endif
    
        /* broadcast n */
		/*
		   Insert call to MPI_Bcast: Broadcast of n
		*/

    fTimeStart = GetRealTime();

    /* the calculation is done here*/
    fPi = CalcPi(n, iMyRank, iNumProcs);

    fLocalPi = fPi;
	/*
	   Insert call to MPI_Reduce: Store global result (sum) in fPi, use local Results from fLocalPi
	*/

    fTimeEnd = GetRealTime();		

    if (iMyRank == 0)
    {
        printf("\npi is approximately = %.20f \nError               = %.20f\n",
               fPi, fabs(fPi - fPi25DT));
        printf(  "wall clock time     = %.20f\n", fTimeEnd - fTimeStart);
    }
	/*
	   Insert cal to MPI_Finalize
	*/
    return 0;
}


double f(double a)
{
    return (4.0 / (1.0 + a*a));
}

double CalcPi (int n, int iRank, int iNumProcs)
{
    const double fH   = 1.0 / (double) n;
    double fSum = 0.0;
    double fX;
    int i;

    for (i = iRank; i < n; i += iNumProcs)
    {
        fX = fH * ((double)i + 0.5);
        fSum += f(fX);
    }
    return fH * fSum;
}
